import unittest

from pretty_cron import prettify_cron as pc


class PrettyCronTest(unittest.TestCase):
    def test_yearly(self):
        assert pc("0 0 1 1 *") == "At 00:00 on the 1st of January"

    def test_one_day_in_month(self):
        assert pc("0 0 1 * *") == "At 00:00 on the 1st of every month"

    def test_one_day_in_month_2nd(self):
        assert pc("0 0 2 * *") == "At 00:00 on the 2nd of every month"

    def test_one_day_in_month_11th(self):
        assert pc("0 0 11 * *") == "At 00:00 on the 11th of every month"

    def test_one_day_in_month_21st(self):
        assert pc("0 0 21 * *") == "At 00:00 on the 21st of every month"

    def test_every_day_in_month(self):
        assert pc("12 15 * 1 *") == "At 15:12 every day in January"

    def test_every_specific_day_in_month(self):
        assert pc("0 0 * 1 1") == "At 00:00 on every Monday in January"

    def test_weekly(self):
        assert pc("0 0 * * 0") == "At 00:00 every Sunday"

    def test_monthly_and_weekly(self):
        assert (
            pc("0 0 1 * 1") ==
            "At 00:00 on the 1st of every month and every Monday"
        )

    def test_every_specific_day_in_month_and_weekly(self):
        assert (
            pc("0 0 1 1 1") ==
            "At 00:00 on the 1st of January and on every Monday in January"
        )

    def test_daily(self):
        assert pc("0 0 * * *") == "At 00:00 every day"

    def test_hourly(self):
        assert pc("0 * * * *") == "At 0 minutes past every hour of every day"

    def test_minutely(self):
        assert (
            pc("* 5 * * *") == "Every minute between 05:00 and 05:59 every day"
        )

    def test_continuous(self):
        assert pc("* * * * *") == "Every minute of every day"

    def test_unsupported(self):
        assert pc("* */6 * * *") == "* */6 * * *"

    def test_invalid_unchanged(self):
        assert pc('* * * * * *') == '* * * * * *'

    def test_nonsense_unchanged(self):
        assert pc('Lalalala') == 'Lalalala'



import datetime


def prettify_cron(expression):
    pieces = []
    for piece in expression.split(" "):
        if piece != "*":
            try:
                piece = int(piece)
            except ValueError:
                # */2 and other cron expressions aren't supported yet - return
                # as-is
                return expression
        pieces.append(piece)

    try:
        minute, hour, month_day, month, week_day = pieces
    except ValueError:
        # More or fewer pieces than expected - return as-is
        return expression

    date = _pretty_date(month_day, month, week_day)
    time = _pretty_time(minute, hour)

    return " ".join(
        filter(None, (time, date))
    )


def _pretty_date(month_day, month, week_day):

    if month_day == "*" and week_day == "*":
        pretty_date = "every day"

        if month != '*':
            pretty_date += " in {0}".format(_human_month(month))
    else:
        if month_day != "*":
            month_day_date = "on the {0}".format(_ordinal(month_day))
        else:
            month_day_date = ""

        if week_day != "*":
            week_day_date = "every {0}".format(_human_week_day(week_day))
        else:
            week_day_date = ""

        if month_day_date:
            if month != "*":
                month_day_date += " of {0}".format(_human_month(month))
            else:
                month_day_date += " of every month"

        if week_day_date and month != "*":
            week_day_date = "on {0} in {1}".format(
                week_day_date,
                _human_month(month)
            )

        pretty_date = " and ".join(
            filter(None, (month_day_date, week_day_date))
        )

    return pretty_date


def _human_month(month):
    return datetime.date(2014, month, 1).strftime("%B")


_WEEKDAYS = {
    0: "Sunday",
    1: "Monday",
    2: "Tuesday",
    3: "Wednesday",
    4: "Thursday",
    5: "Friday",
    6: "Saturday",
}


def _human_week_day(day):
    return _WEEKDAYS[day]


_ORDINAL_SUFFIXES = {
    1: 'st',
    2: 'nd',
    3: 'rd'
}


def _ordinal(n):
    if 10 <= (n % 100) < 20:
        suffix = 'th'
    else:
        suffix = _ORDINAL_SUFFIXES.get(n % 10, 'th')

    return str(n) + suffix


def _pretty_time(minute, hour):
    if minute != "*" and hour != "*":
        the_time = datetime.time(hour=hour, minute=minute)
        pretty_time = "At {0}".format(the_time.strftime("%H:%M"))

    elif minute != "*" and hour == '*':
        pretty_time = "At {0} minutes past every hour of".format(minute)

    elif minute == "*" and hour != '*':
        start_time = datetime.time(hour=hour)
        end_time = datetime.time(hour=hour, minute=59)

        pretty_time = "Every minute between {0} and {1}".format(
            start_time.strftime("%H:%M"),
            end_time.strftime("%H:%M"),
        )
    else:
        pretty_time = "Every minute of"

    return pretty_time



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

try:
    from setuptools import setup
    from setuptools.command.test import test as TestCommand
except ImportError:
    from distutils.core import setup
    from distutils.command.test import test as TestCommand


def get_version(package):
    """
    Return package version as listed in `__version__` in `__init__.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


version = get_version('pretty_cron')


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep wheel"):
        print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name='pretty-cron',
    version=version,
    description="Converts crontab expressions to human-readable descriptions.",
    long_description=readme + '\n\n' + history,
    author="Adam Johnson",
    author_email='me@adamj.eu',
    url='https://github.com/adamchainz/pretty-cron',
    packages=get_packages('pretty_cron'),
    include_package_data=True,
    install_requires=[],
    license="MIT",
    zip_safe=False,
    keywords='cron',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
    tests_require=['pytest'],
    cmdclass={'test': PyTest},
)
